#ident "@(#)head/sys:gc.h	1.6"
#define PORTS_SHIFT		4
#define NUM_PORTS		(1<<PORTS_SHIFT)
#define SYNC_PORTS		8

#define GC_BASE(slot)		(unsigned char *)(0x600000 + (slot << 16))
#define GC_TOP(slot)		(unsigned char *)(0x60fffc + (slot << 16))
#define GC_VREGS(slot)		(int *)(0x60ff00 + (slot << 16))
#define GC_RTB(slot)		(union rtb *)(0x60fffc + (slot << 16))
#define GC_BDVECT(slot)		(struct bd_desc *)(0x60ffec + (slot << 16))
#define GC_SAR(slot)		(int *)(0x60ff80 + (slot << 16))

/*
 *	protocol numbers
 */

#define GC_CNTRL		0x0
#define GC_TTY			0x1
#define GC_LP			0x2
#define GC_TTM			0x3
#define GC_BSC			0x4
#define GC_SNA			0x5
#define GC_SDLC			0x6
#define GC_MON			0xf

/*
 *	master to slave command groups
 */

#define GC_GC_COMMANDS		(GC_CTRL<<12	/* gc command group	*/
#define GC_TTY_COMMANDS		(GC_TTY<<12)	/* tty command group	*/
#define GC_LP_COMMANDS		(GC_LP<<12)	/* lp command group	*/
#define GC_TTM_COMMANDS		(GC_TTM<<12)	/* ttm command group	*/
#define GC_BSC_COMMANDS		(GC_BSC<<12)	/* bsc command group	*/
#define GC_SNA_COMMANDS		(GC_SNA<<12)	/* sna command group	*/
#define GC_SDLC_COMMANDS	(GC_SDLC<<12)	/* sdlc command group	*/
#define GC_MON_COMMANDS		(GC_MON<<12)	/* mon command group	*/

struct cir_buf {
	unsigned char *	cb_hoq;
	unsigned char *	cb_eoq;
	unsigned char *	cb_base;
	unsigned char *	cb_end;
	unsigned short	cb_hi_water;
	unsigned short	cb_low_water;
	unsigned short	cb_hog_water;
	unsigned char cb_in_flag;
	unsigned char cb_out_flag;
};

struct cirbuf {
	unsigned short	flag;
	unsigned short	count;
	unsigned char *	hoq;
	unsigned char *	eoq;
	unsigned char *	base;
	unsigned char *	end;
	unsigned short	hi_water;
	unsigned short	low_water;
	unsigned short	hog_water;
};

struct gc {
	struct cir_buf 	*gc_mtos;
	struct cir_buf 	*gc_stom;
	unsigned char 	*gc_base;
	int	 	*gc_vreg;
	union rtb	*gc_rtb;
	int		*gc_tas;
	long		*gc_sar;
	int		 gc_code;
	unsigned char	 gc_flag;
	unsigned char	 gc_board;
	int 		(* gc_reset)();
};

struct gc_switch {
	unsigned short	  gcs_group;		/* command group	*/
	int		(*gcs_initialize)();	/* initialize routine	*/
	int		(*gcs_interrupt)();	/* interrupt routine	*/
	int		(*gcs_reset)();		/* reset routine	*/
};

/*  cirbuf flags */

#define BUFEMPTY	0x0100		/* buffer empty */
#define BUFFULL		0x0200		/* buffer full */

/*
	virtual registers
 */

#define GC_VR_MTOS		0	/* master to slave commands	*/
#define GC_VR_STOM		1	/* slave to master commands	*/
#define GC_VR_TAS		2	/* test and set owner		*/
#define GC_VR_CODE		3	/* code running on gcp		*/
#define GC_VR_TIME		4	/* time of day (sec)		*/
#define GC_VR_SAR		5	/* system activity counters	*/
#define GC_VR_MON		8	/* monitor control blocks	*/
#define GC_VR_TTY		9	/* tty control blocks		*/
#define GC_VR_LP		10	/* lp control blocks		*/
#define GC_VR_TTM		11	/* ttm control blocks		*/
#define GC_VR_MUXT		12	/* mux trace control block	*/
#define GC_VR_BSC		13	/* bsc control blocks		*/
#define GC_VR_SNA		14	/* sna control blocks		*/
#define GC_VR_SDLC		15	/* sdlc control blocks		*/

/*
	gc_tas test and set owner
 */

#define LOCK_FREE	0	/* channel not locked */
#define LOCK_MASTER	1	/* master has channel locked */
#define LOCK_SLAVE	2	/* slave has channel locked */
#define LOCK_ERROR	3	/* channel lock error */

#define GC_INT_DEV(n) (n&0x00ff)
#define GC_INT_GRP(n) (n&0xf000)
#define GC_INT_CMD(n) (n&0xff00)
